import sqlite3
from pprint import pprint
from datetime import date, timedelta
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import sys
from sklearn import linear_model 
from sklearn.neighbors import KNeighborsClassifier

#------------------------------------------------
#------------------------------------------------
def getFeatures(db, sym, xdata, nfeatures, windays):
    c = db.cursor()
    flist = []
    for row in xdata:
        yesterday = row - timedelta(days=1)
        lowdate = row - timedelta(days=(windays+1))
        t = \
        (sym,lowdate.isoformat(),yesterday.isoformat(),nfeatures)
        c.execute('SELECT open FROM stock WHERE sym=? AND sdate >= ? \
                AND sdate <= ? ORDER BY sdate DESC LIMIT ?', t)
        tlist = c.fetchall()
        f = [ t[0] for t in tlist ]
        flist.append(f) 

    return flist

def getMeanAbsErr(predicted, actual):
    n = len(predicted)
    totalerr = 0
    for i in range(n):
        totalerr = totalerr + abs(predicted[i]-actual[i])
    return (totalerr/n)

def getKNNClassifier(db, sym, start, end, kneighbors, nfeatures, windays):
    t = (sym,start,end)
    c.execute('SELECT sdate, open FROM stock WHERE sym=? AND sdate >= ? \
                AND sdate <= ? ORDER BY sdate', t)
    stkdata = c.fetchall()

    xlist = []
    ylist = []
    for row in stkdata:
        sdate = str2date(row[0])
        xlist.append(sdate)
        ylist.append(int(row[1]))

    flist = getFeatures(db, sym, xlist, nfeatures, windays)
    myknn = \
        KNeighborsClassifier(n_neighbors=kneighbors,\
            algorithm='kd_tree', p=2, weights='distance')

    myknn.fit(flist, ylist)
    return myknn


#------------------------------------------------
#------------------------------------------------
def getLinReg(xdata, ydata):
    # massage x data into the input format for linear
    # regression
    xd = [[x.toordinal()] for x in xdata ]

    # create linear regression object
    regr = linear_model.LinearRegression()

    # least squares fit
    regr.fit(xd, ydata)

    # pprint(regr.coef_)
    return regr.predict(xd)
    
#------------------------------------------------
#------------------------------------------------
def str2date(datestr):
    (y,m,d) = datestr.split('-')
    return date(int(y), int(m), int(d))

#------------------------------------------------
#------------------------------------------------
def getMovingAvg(db, sym, data, windays):
    movavg = []
    c = db.cursor()
    for row in data:
        curdate = str2date(row[0])
        lowdate = curdate - timedelta(days=windays)
        t = (sym,lowdate,curdate)
        c.execute('SELECT AVG(open)  FROM stock WHERE sym=? AND sdate >= ? \
                AND sdate <= ?', t)
        (avg,) = c.fetchone()
        movavg.append(avg) 

    return movavg


#------------------------------------------------
# MAIN
#------------------------------------------------
if len(sys.argv)!=2:
   print('Usage: plotdata.py <Symbol>')
   sys.exit()

sym = sys.argv[1]

# we will plot the data for this date range
start = date(2016,8,1)
end   = date(2017,8,1)

# KNN training parameters
kneighbors = 3
nfeatures = 5
windays = 12
trgstart = date(2015,8,1)
trgend   = date(2016,8,1)

conn = sqlite3.connect('stock.db')

c = conn.cursor()

t = (sym,start.isoformat(),end.isoformat())
c.execute('SELECT * FROM stock WHERE sym=? AND sdate >= ? \
                AND sdate <= ? ORDER BY sdate', t)
stkdata = c.fetchall()

ylist = [ row[1] for row in stkdata ]
xlist = [ str2date(row[0]) for row in stkdata ]

# get linear regression trend line
lineartrend = getLinReg(xlist, ylist)

# get moving average curves
ma30 = getMovingAvg(conn,sym,stkdata,30)
ma60 = getMovingAvg(conn,sym,stkdata,60)

# get KNN prediction curves 
myknn = getKNNClassifier(conn, sym, trgstart, trgend, \
                kneighbors, nfeatures, windays)

knnflist = getFeatures(conn, sym, xlist, nfeatures, windays)
knnpredicted = myknn.predict(knnflist)
 
#pprint(lineartrend)
#sys.exit()
#pprint(xlist)
#pprint(ylist)

years = mdates.YearLocator()   # every year
months = mdates.MonthLocator()  # every month
days = mdates.DayLocator(interval=14)
DateFmt = mdates.DateFormatter('%Y-%m-%d')

fig, ax = plt.subplots()
ax.plot(xlist, ylist, label='open')
ax.plot(xlist, lineartrend, label='linear')
ax.plot(xlist, knnpredicted, label='knn prediction')
ax.plot(xlist, ma30, label='30d MA')
ax.plot(xlist, ma60, label='60d MA')


ax.legend(loc='upper left', frameon=False)

# format the ticks
ax.xaxis.set_major_locator(months)
ax.xaxis.set_major_formatter(DateFmt)
ax.xaxis.set_minor_locator(days)

datemin = xlist[0]
datemax = xlist[len(xlist)-1]
ax.set_xlim(datemin, datemax)

# format the coords message box
def price(x):
    return '$%1.2f' % x

ax.format_xdata = mdates.DateFormatter('%Y-%m-%d')
ax.format_ydata = price
ax.grid(True)

# rotates and right aligns the x labels, and moves the
# bottom of the
# axes up to make room for them
fig.autofmt_xdate()

plt.show()

conn.close()


